<?php

/** --------------------------------------------------------------------------------
 * This repository manages the creation of a new tenant database and also populating
 * it
 *
 * @package    Grow CRM
 * @author     NextLoop
 *----------------------------------------------------------------------------------*/

namespace App\Repositories\Landlord;
use DB;
use Exception;
use Illuminate\Support\Facades\Http;
use Log;
use Spatie\Multitenancy\Models\Tenant;

class DatabaseRepository {

    /**
     * Inject dependecies
     */
    public function __construct() {

    }

    /**
     * create a new database
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function createDatabase() {

        Log::info("creating a new tenant database - started", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //database name
        switch (env('DB_METHOD')) {
        case 'mysql_user':
            return $this->tenantDatabaseMysqlUser();
        case 'cpanel':
            return $this->tenantDatabaseCpanel();
        default:
            return false;
            Log::error("creating a new tenant database - failed - a valid (DB_METHOD) was not found in the .env file", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
        }
    }

    /**
     * delete a new database
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteDatabase($database_name) {

        Log::info("deleteing a database ($database_name) - started", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //database name
        switch (env('DB_METHOD')) {
        case 'mysql_user':
            return $this->deleteDatabaseDirect($database_name);
        case 'cpanel':
            return true; //TEMP TODO
            //return $this->deleteDatabaseCpanel($database_name);
        default:
            return false;
            Log::error("deleteting database ($database_name) - failed - a valid (DB_METHOD) was not found in the .env file", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
        }
    }

    /**
     * full process for creating mysql database during runtime for a tenant
     *  - create a new database
     *  - grant permissions on the database
     *
     * @return mixed database name or false
     */
    public function tenantDatabaseMysqlUser() {

        Log::info("create tenant database (method: mysql_user)  - started", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //create the database
        if (!$database_name = $this->createDatabaseDirect(env('DB_METHOD_PREFIX'))) {
            Log::error("create tenant database failed - database could not be created", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
            return false;
        }

        //add user to database - does not seem to be required because the mysql user that created the database already has permissions on the database
        /*
        if (!$this->grantDatabaseDirect($database_name, env('DB_USERNAME'), env('DB_HOST'))) {
        Log::error("create tenant database failed - granting database permission failed", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
        return false;
        }
         */

        //all ok
        Log::info("create tenant database ($database_name) - completed", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
        return $database_name;

    }

    /**
     * full process for creating mysql database during runtime for a tenant
     *  - create a new database
     *  - grant permissions on the database
     *
     * @return mixed database name or false
     */
    public function tenantDatabaseCpanel() {

        Log::info("create tenant database (method: cpanel) - started", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //create a new database
        $data = [
            'cpanel_api_key' => env('DB_METHOD_CPANEL_API_KEY'),
            'cpanel_username' => env('DB_METHOD_CPANEL_USERNAME'),
            'cpanel_url' => env('DB_METHOD_CPANEL_URL'),
        ];
        if (!$database_name = $this->createDatabaseCpanel($data)) {
            Log::error("create tenant database - failed - database could not be created", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
            return false;
        }

        //grant privileges on database
        $data['mysql_database'] = $database_name;
        $data['mysql_user'] = env('DB_METHOD_CPANEL_USER');
        if (!$this->privilegesCpanelDatabase($data)) {
            Log::error("create tenant database - failed - database could not be created", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
            return false;
        }

        //all ok
        Log::info("create tenant database ($database_name) - completed", ['process' => '[database-repository]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
        return $database_name;
    }

    /**
     * create a new database using the 'user/direct' method
     *  - database name and prefix must have maximum of 64 characters
     *
     * @param string prefix database prefix
     * @return mixed database name or false
     */
    public function createDatabaseDirect($prefix = '') {

        //database name
        $database_name = databaseName($prefix);

        Log::info("creating database started", ['process' => '[database-repository][direct]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //create a new database
        try {
            DB::connection('tenant')->statement("CREATE DATABASE $database_name");
        } catch (Exception $e) {
            Log::critical("creating database failed (" . $e->getMessage() . ")", ['process' => '[create-tenant-database][direct]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'database_name' => $database_name, 'tenant_id' => $customer->tenant_id]);
            return false;
        }

        Log::info("database ($database_name) created", ['process' => '[database-repository][direct]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //return
        return $database_name;

    }


    /**
     * delete a database from the server
     *
     * @param string database_name db name
     * @return bool
     */
    public function deleteDatabaseDirect($database_name = '') {

        //valiate
        if ($database_name == '') {
            Log::error("deleting a database failed - no database name was provided", ['process' => '[database-repository][direct]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
        }

        //delete a new database
        try {
            DB::statement('DROP DATABASE IF EXISTS ' . $database_name);
        } catch (Exception $e) {
            Log::critical("deleting a database ($database_name) failed - (" . $e->getMessage() . ")", ['process' => '[create-tenant-database][direct]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'database_name' => $database_name]);
            return false;
        }

        Log::info("deleting database ($database_name) - completed", ['process' => '[database-repository][direct]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);

        //return
        return true;

    }

    /**
     * create a new database using the Cpanel UAPI https://api.docs.cpanel.net/cpanel/tokens/
     *  - database name and prefix must have maximum of 64 characters
     *
     * @param array data
     * @return mixed database name or false
     */
    public function createDatabaseCpanel($data = '') {

        //validation
        if (!isset($data['cpanel_api_key'])) {
            Log::error("creating database failed - Cpanel API key is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['cpanel_username'])) {
            Log::error("creating database failed - Cpanel user name is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['cpanel_url'])) {
            Log::error("creating database failed - Cpanel URL is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //database name
        $database_name = databaseName($data['cpanel_username'] . '_');
        $database_name = strtolower($database_name);

        Log::info("creating database started", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);

        //cpanel api details
        $cpanel_username = $data['cpanel_username'];
        $cpanel_api_key = $data['cpanel_api_key'];
        $url = cpanelCleanURL($data['cpanel_url']);
        $cpanel_url = "$url/execute/Mysql/create_database";

        //attempt to create the databse
        try {

            //connect to cpnel UAPI
            $response = Http::withHeaders([
                'Authorization' => "cpanel $cpanel_username:$cpanel_api_key",
            ])->withoutVerifying()->get($cpanel_url, [
                'name' => $database_name,
            ]);

            //get the json payload
            $response_data = $response->json();

            //check results
            if ($response->successful()) {
                if (isset($response_data['status']) && $response_data['status'] == 1) {
                    Log::info("creating database ($database_name) completed", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
                    //success - return database name
                    return $database_name;
                }
            }

            //api error
            Log::error("creating database failed - cpanel api error", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'errors' => $response]);
            return false;

        } catch (Exception $e) {
            Log::error("creating database failed - " . $e->getMessage(), ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //error
        return false;
    }

    /**
     * create a new mysql user in Cpanel, using UAPI https://api.docs.cpanel.net/cpanel/tokens/
     * mysql username will be prefixed by the cpanel user name
     *
     * @param array data
     * @return mixed database name or false
     */
    public function createMySQLUserCpanel($data = '') {

        //validation
        if (!isset($data['cpanel_api_key'])) {
            Log::error("creating mysql user failed - Cpanel API key is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['cpanel_username'])) {
            Log::error("creating mysql user failed - Cpanel user name is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['cpanel_url'])) {
            Log::error("creating mysql user failed - Cpanel URL is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //database name
        $mysql_user = cpanelMysqlUsername($data['cpanel_username']);
        $mysql_password = generatePassword(12);

        Log::info("creating mysql user started", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);

        //cpanel api details
        $cpanel_username = $data['cpanel_username'];
        $cpanel_api_key = $data['cpanel_api_key'];
        $url = cpanelCleanURL($data['cpanel_url']);
        $cpanel_url = "$url/execute/Mysql/create_user";

        //attempt to create the mysql user
        try {

            //connect to cpnel UAPI
            $response = Http::withHeaders([
                'Authorization' => "cpanel $cpanel_username:$cpanel_api_key",
            ])->withoutVerifying()->get($cpanel_url, [
                'name' => $mysql_user,
                'password' => $mysql_password,
            ]);

            //get the json payload
            $response_data = $response->json();

            //check results
            if ($response->successful()) {
                if (isset($response_data['status']) && $response_data['status'] == 1) {
                    Log::info("creating mysql user ($mysql_user) completed", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
                    //success - return database name
                    return [
                        'username' => $mysql_user,
                        'password' => $mysql_password,
                    ];
                }
            }

            //api error
            Log::error("creating mysql user failed - cpanel api error", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'errors' => $response]);
            return false;

        } catch (Exception $e) {
            Log::error("creating mysql user failed - " . $e->getMessage(), ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //error
        return false;
    }

    /**
     * create a new mysql user in Cpanel, using UAPI https://api.docs.cpanel.net/cpanel/tokens/
     * mysql username will be prefixed by the cpanel user name
     *
     * @param array data
     * @return mixed database name or false
     */
    public function privilegesCpanelDatabase($data = '') {

        //validation
        if (!isset($data['cpanel_api_key'])) {
            Log::error("settings mysql privileges failed - Cpanel API key is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['cpanel_username'])) {
            Log::error("settings mysql privileges failed - Cpanel user name is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['cpanel_url'])) {
            Log::error("settings mysql privileges failed - Cpanel URL is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['mysql_user'])) {
            Log::error("settings mysql privileges failed - Mysql user is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //validation
        if (!isset($data['mysql_database'])) {
            Log::error("settings mysql privileges failed - mysql database is missing", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //database name
        $mysql_user = $data['mysql_user'];
        $mysql_database = $data['mysql_database'];

        Log::info("settings mysql privileges on ($mysql_database) started", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
        Log::info("note - this process requires a valid 'cpanel mysql user' see .env file for details", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);

        //cpanel api details
        $cpanel_username = $data['cpanel_username'];
        $cpanel_api_key = $data['cpanel_api_key'];
        $url = cpanelCleanURL($data['cpanel_url']);
        $cpanel_url = "$url/execute/Mysql/set_privileges_on_database";

        //attempt to create the mysql user
        try {

            //connect to cpnel UAPI
            $response = Http::withHeaders([
                'Authorization' => "cpanel $cpanel_username:$cpanel_api_key",
            ])->withoutVerifying()->get($cpanel_url, [
                'user' => $mysql_user,
                'database' => $mysql_database,
                'privileges' => 'ALL PRIVILEGES',
            ]);

            //get the json payload
            $response_data = $response->json();

            //check results
            if ($response->successful()) {
                if (isset($response_data['status']) && $response_data['status'] == 1) {
                    Log::info("settings mysql privileges ($mysql_database) completed", ['process' => '[database-repository][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__]);
                    //success - return database name
                    return true;
                }
            }

            //api error
            Log::error("settings mysql privileges failed - cpanel api error", ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'errors' => $response]);
            return false;

        } catch (Exception $e) {
            Log::error("settings mysql privileges failed - " . $e->getMessage(), ['process' => '[create-tenant-database][cpanel]', config('app.debug_ref'), 'function' => __function__, 'file' => basename(__FILE__), 'line' => __line__, 'path' => __file__, 'data' => $data]);
            return false;
        }

        //error
        return false;
    }
}